from social_core.backends.oauth import BaseOAuth2PKCE
from social_core.backends.open_id_connect import OpenIdConnectAuth


class MichiganOnlineOAuth2(BaseOAuth2PKCE):
    """Backend for Michigan Online OAuth2"""

    name = "michiganonline-oauth"
    ACCESS_TOKEN_METHOD = "POST"
    DEFAULT_SCOPE = ["profile", "email"]
    PKCE_DEFAULT_CODE_CHALLENGE_METHOD = "S256"

    def base_url(self):
        return self.setting("BASE_URL", "https://online.umich.edu/")

    def authorization_url(self):
        return f"{self.base_url()}o/authorize/"

    def access_token_url(self):
        return f"{self.base_url()}o/token/"

    def user_data(self, access_token, *args, **kwargs):
        return self.get_json(
            f"{self.base_url()}/api/account/info.json",
            headers={"Authorization": f"Bearer {access_token}"},
        )

    def get_user_details(self, response):
        return {
            "username": response.get("username"),
            "email": response.get("email"),
            "first_name": response.get("first_name"),
            "last_name": response.get("last_name"),
            "fullname": response.get("full_name"),
            "image_url": response.get("image_url"),
        }


class MichiganOnlineOIDC(OpenIdConnectAuth):
    """Authentication backend for Michigan Online OIDC."""

    name = "michiganonline-oidc"

    def oidc_endpoint(self):
        return self.setting("ENDPOINT", "https://online.umich.edu/o")
